@@ -1,4 +1,5 @@
 import socket
+import wrapt  # Required for monkey patching socket.SocketIO.readinto
 import logging
 import re
 import itertools
@@ -9,6 +10,29 @@ from .scpi_resource import ScpiResource
 from .scpi_exceptions import ScpiTransportException
 
 
+def readinto_monkey(self, b=None):
+    """For some unknown reason original socket.SocketIO.readinto function
+    is designed to bail on successive calls once a timeout has occured.
+    Override this functionality, so that the file object keeps working,
+    since timeouts may be tolerable in SCPI."""
+    self._checkClosed()
+    self._checkReadable()
+    while True:
+        try:
+            return self._sock.recv_into(b)
+        except socket.timeout:
+            raise
+        except socket.error as e:
+            if e.args[0] in socket._blocking_errnos:
+                return None
+            raise
+
+
+@wrapt.patch_function_wrapper(socket.SocketIO, 'readinto')
+def readinto_wrap(wrapped, instance, args, kwargs):
+    return readinto_monkey(instance, *args, **kwargs)
+
+
 class ScpiTcpIpTransport(ScpiTransportBase):
     DEFAULT_PORT: int = 5025
 
@@ -38,7 +62,6 @@ class ScpiTcpIpTransport(ScpiTransportBase):
             def add_service(self, zeroconf, zc_type, zc_name):
                 self.services.update({zc_name: zeroconf.get_service_info(zc_type, zc_name)})
 
-
             def update_service(self, zeroconf, zc_type, zc_name):
                 self.services.update({zc_name: zeroconf.get_service_info(zc_type, zc_name)})
 
@@ -72,14 +95,15 @@ class ScpiTcpIpTransport(ScpiTransportBase):
                              serialnum=service.properties[b'SerialNumber'].decode(
                                  'utf-8') if b'SerialNumber' in service.properties else None,
                              info=service
-                             ) for service in listener.services.values() for parsed_address in service.parsed_scoped_addresses()
+                             ) for service in listener.services.values() for parsed_address
+                in service.parsed_scoped_addresses()
                 if any([pattern.fullmatch(service.get_name()) for pattern in patterns])]
 
     @classmethod
     def from_resource_name(cls, resource_name: str) -> Optional[ScpiResource]:
         m = re.match((
             r'^(?P<prefix>(?P<type>TCPIP)(?P<board>\d+)?)'
-            r'::((?P<host_address>[^:]+)|(\[(?P<host_address6>.+)\]))'
+            r'::((?P<host_addr>[^:]+)|(\[(?P<host_addr6>.+)\]))'
             r'::(?P<host_port>([0-9]+))'
             r'(::(?P<suffix>SOCKET))?$'
         ), resource_name, re.IGNORECASE)
@@ -91,7 +115,7 @@ class ScpiTcpIpTransport(ScpiTransportBase):
         groupdict = m.groupdict()
 
         # Get either IPv4 or IPv6 address
-        host_address = groupdict['host_address'] if groupdict['host_address'] is not None else groupdict['host_address6']
+        host_address = groupdict['host_addr'] if groupdict['host_addr'] is not None else groupdict['host_addr6']
         host_port = groupdict['host_port']
 
         return ScpiResource(
@@ -101,12 +125,13 @@ class ScpiTcpIpTransport(ScpiTransportBase):
 
     @classmethod
     def to_resource_name(cls, resource: ScpiResource) -> str:
-        host_address = resource.address[0] if not ':' in resource.address[0] else f'[{resource.address[0]}]'
+        host_address = resource.address[0] if ':' not in resource.address[0] else f'[{resource.address[0]}]'
         host_port = resource.address[1] if len(resource.address) > 1 else cls.DEFAULT_PORT
 
         return f'TCPIP::{host_address}::{host_port}::SOCKET'
 
-    def __init__(self, address: Union[tuple, str], timeout: float=5.0, ip_family=socket.AF_UNSPEC, **kwargs):
+    def __init__(self, address: Union[tuple, str], timeout: float = 5.0,
+                 ip_family: socket.AddressFamily = socket.AF_UNSPEC, **kwargs):
         super().__init__(**kwargs)
         self._logger = logging.getLogger(__name__)
         self._sock = None
@@ -114,13 +139,14 @@ class ScpiTcpIpTransport(ScpiTransportBase):
         if isinstance(address, str):
             # Convert from string to tuple first
             addr = urlparse('//' + address)
-            if (addr.hostname is None):
+            if addr.hostname is None:
                 raise ScpiTransportException(f'{address} is not a valid hostname')
             address = (addr.hostname, addr.port) if addr.port is not None else (addr.hostname, self.DEFAULT_PORT)
 
         try:
             # Get address info
-            addr_infos = socket.getaddrinfo(address[0], address[1], family=ip_family, type=socket.SOCK_STREAM, proto=socket.IPPROTO_TCP)
+            addr_infos = socket.getaddrinfo(address[0], address[1], family=ip_family,
+                                            type=socket.SOCK_STREAM, proto=socket.IPPROTO_TCP)
         except socket.gaierror as e:
             raise ScpiTransportException(f'Host {address[0]} is unknown ({str(e)})')
 
